@@ -30,8 +30,9 @@ def install_deps():
     print('Installing kitty dependencies...')
     sys.stdout.flush()
     if is_macos:
-        items = (x.split()[1].strip('"') for x in open('Brewfile').readlines() if x.strip().startswith('brew '))
+        items = [x.split()[1].strip('"') for x in open('Brewfile').readlines() if x.strip().startswith('brew ')]
         openssl = 'openssl'
+        items.remove('go')  # already installed by ci.yml
         import ssl
         if ssl.OPENSSL_VERSION_INFO[0] == 1:
             openssl += '@1.1'
@@ -129,6 +130,12 @@ def main():
         package_kitty()
     elif action == 'test':
         test_kitty()
+    elif action == 'gofmt':
+        q = subprocess.check_output('gofmt -s -l tools'.split())
+        if q.strip():
+            q = '\n'.join(filter(lambda x: not x.rstrip().endswith('_generated.go'), q.decode().strip().splitlines())).strip()
+            if q:
+                raise SystemExit(q)
     else:
         raise SystemExit(f'Unknown action: {action}')
 
