@@ -6,13 +6,21 @@ import re
 import sys
 from contextlib import suppress
 from typing import (
-    TYPE_CHECKING, Callable, Dict, Iterator, List, NamedTuple, Optional, Tuple
+    TYPE_CHECKING,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    NamedTuple,
+    Optional,
+    Tuple,
 )
 
 from kitty.cli import parse_args
 from kitty.cli_stub import AskCLIOptions
 from kitty.constants import cache_dir
 from kitty.fast_data_types import truncate_point_for_length, wcswidth
+from kitty.types import run_once
 from kitty.typing import BossType, KeyEventType, TypedDict
 from kitty.utils import ScreenSize
 
@@ -158,14 +166,17 @@ def extra_for(width: int, screen_width: int) -> int:
 
 class Password(Handler):
 
-    def __init__(self, cli_opts: AskCLIOptions, prompt: str) -> None:
+    def __init__(self, cli_opts: AskCLIOptions, prompt: str, is_password: bool = True, initial_text: str = '') -> None:
         self.cli_opts = cli_opts
         self.prompt = prompt
+        self.initial_text = initial_text
         from kittens.tui.line_edit import LineEdit
-        self.line_edit = LineEdit(is_password=True)
+        self.line_edit = LineEdit(is_password=is_password)
 
     def initialize(self) -> None:
         self.cmd.set_cursor_shape('beam')
+        if self.initial_text:
+            self.line_edit.on_text(self.initial_text, True)
         self.draw_screen()
 
     @Handler.atomic_update
@@ -205,7 +216,7 @@ class Password(Handler):
         return ''
 
 
-class Choose(Handler):
+class Choose(Handler):  # {{{
     mouse_tracking = MouseTracking.buttons_only
 
     def __init__(self, cli_opts: AskCLIOptions) -> None:
@@ -441,6 +452,18 @@ class Choose(Handler):
     def on_interrupt(self) -> None:
         self.quit_loop(1)
     on_eot = on_interrupt
+# }}}
+
+
+@run_once
+def init_readline() -> None:
+    import readline
+    with suppress(OSError):
+        readline.read_init_file()
+    if 'libedit' in readline.__doc__:
+        readline.parse_and_bind("bind ^I rl_complete")
+    else:
+        readline.parse_and_bind('tab: complete')
 
 
 def main(args: List[str]) -> Response:
@@ -450,7 +473,7 @@ def main(args: List[str]) -> Response:
     global readline
     msg = 'Ask the user for input'
     try:
-        cli_opts, items = parse_args(args[1:], option_text, '', msg, 'kitty ask', result_class=AskCLIOptions)
+        cli_opts, items = parse_args(args[1:], option_text, '', msg, 'kitty +kitten ask', result_class=AskCLIOptions)
     except SystemExit as e:
         if e.code != 0:
             print(e.args[0])
@@ -472,17 +495,22 @@ def main(args: List[str]) -> Response:
         loop.loop(phandler)
         return {'items': items, 'response': phandler.response}
 
-    import readline as rl
-    readline = rl
-    from kitty.shell import init_readline
-    init_readline()
-    response = None
+    # we do this file descriptor dance to get readline to work even when STDOUT
+    # is redirected
+    orig_stdout = os.dup(sys.stdout.fileno())
+    try:
+        with open(os.ctermid(), 'r') as tty:
+            os.dup2(tty.fileno(), sys.stdin.fileno())
+        with open(os.ctermid(), 'w') as tty:
+            os.dup2(tty.fileno(), sys.stdout.fileno())
+        import readline as rl
+        readline = rl
+        init_readline()
+        response = None
 
-    with alternate_screen(), HistoryCompleter(cli_opts.name):
-        if cli_opts.message:
-            print(styled(cli_opts.message, bold=True))
-
-        with suppress(KeyboardInterrupt, EOFError):
+        with alternate_screen(), HistoryCompleter(cli_opts.name), suppress(KeyboardInterrupt, EOFError):
+            if cli_opts.message:
+                print(styled(cli_opts.message, bold=True))
             if cli_opts.default:
                 def prefill_text() -> None:
                     readline.insert_text(cli_opts.default or '')
@@ -492,6 +520,10 @@ def main(args: List[str]) -> Response:
                 readline.set_pre_input_hook()
             else:
                 response = input(prompt)
+        sys.stdout.flush()
+        os.dup2(orig_stdout, sys.stdout.fileno())
+    finally:
+        os.close(orig_stdout)
     return {'items': items, 'response': response}
 
 
@@ -505,4 +537,5 @@ def handle_result(args: List[str], data: Response, target_window_id: int, boss:
 if __name__ == '__main__':
     ans = main(sys.argv)
     if ans:
-        print(ans)
+        import json
+        print(json.dumps(ans))
