@@ -14,16 +14,24 @@ from enum import Enum, auto
 from functools import partial
 from gettext import gettext as _
 from typing import (
-    Any, DefaultDict, Dict, Iterable, Iterator, List, Optional, Tuple, Union
+    Any,
+    DefaultDict,
+    Dict,
+    Iterable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Union,
 )
 
-from kitty.cli import CONFIG_HELP, parse_args
+from kitty.cli import CONFIG_HELP, CompletionSpec, parse_args
 from kitty.cli_stub import DiffCLIOptions
 from kitty.conf.utils import KeyAction
 from kitty.constants import appname
 from kitty.fast_data_types import wcswidth
 from kitty.key_encoding import EventType, KeyEvent
-from kitty.utils import ScreenSize
+from kitty.utils import ScreenSize, extract_all_from_tarfile_safely
 
 from ..tui.handler import Handler
 from ..tui.images import ImageManager, Placement
@@ -32,21 +40,33 @@ from ..tui.loop import Loop
 from ..tui.operations import styled
 from . import global_data
 from .collect import (
-    Collection, add_remote_dir, create_collection, data_for_path,
-    lines_for_path, sanitize, set_highlight_data
+    Collection,
+    add_remote_dir,
+    create_collection,
+    data_for_path,
+    lines_for_path,
+    sanitize,
+    set_highlight_data,
 )
 from .config import init_config
 from .options.types import Options as DiffOptions
 from .patch import Differ, Patch, set_diff_command, worker_processes
 from .render import (
-    ImagePlacement, ImageSupportWarning, Line, LineRef, Reference, render_diff
+    ImagePlacement,
+    ImageSupportWarning,
+    Line,
+    LineRef,
+    Reference,
+    render_diff,
 )
 from .search import BadRegex, Search
 
 try:
     from .highlight import (
-        DiffHighlight, get_highlight_processes, highlight_collection,
-        initialize_highlighter
+        DiffHighlight,
+        get_highlight_processes,
+        highlight_collection,
+        initialize_highlighter,
     )
     has_highlighter = True
     DiffHighlight
@@ -229,8 +249,8 @@ class DiffHandler(Handler):
         self.diff_lines: Tuple[Line, ...] = tuple(render_diff(self.collection, self.diff_map, self.args, self.screen_size.cols, self.image_manager))
         self.margin_size = render_diff.margin_size
         self.ref_path_map: DefaultDict[str, List[Tuple[int, Reference]]] = defaultdict(list)
-        for i, l in enumerate(self.diff_lines):
-            self.ref_path_map[l.ref.path].append((i, l.ref))
+        for i, dl in enumerate(self.diff_lines):
+            self.ref_path_map[dl.ref.path].append((i, dl.ref))
         self.max_scroll_pos = len(self.diff_lines) - self.num_lines
         if self.current_search is not None:
             self.current_search(self.diff_lines, self.margin_size, self.screen_size.cols)
@@ -319,7 +339,7 @@ class DiffHandler(Handler):
             fg=self.opts.foreground, bg=self.opts.background,
             cursor=self.opts.foreground, select_fg=self.opts.select_fg,
             select_bg=self.opts.select_bg)
-        self.cmd.set_cursor_shape('bar')
+        self.cmd.set_cursor_shape('beam')
 
     def finalize(self) -> None:
         self.cmd.set_default_colors()
@@ -577,6 +597,7 @@ number set in :file:`diff.conf`.
 
 --config
 type=list
+completion=type:file ext:conf group:"Config files" kwds:none,NONE
 {config_help}
 
 
@@ -629,7 +650,7 @@ def get_ssh_file(hostname: str, rpath: str) -> str:
         raise SystemExit(p.returncode)
     with tarfile.open(fileobj=io.BytesIO(raw), mode='r:') as tf:
         members = tf.getmembers()
-        tf.extractall(tdir)
+        extract_all_from_tarfile_safely(tf, tdir)
         if len(members) == 1:
             for root, dirs, files in os.walk(tdir):
                 if files:
@@ -655,6 +676,7 @@ def main(args: List[str]) -> None:
     opts = init_config(cli_opts)
     set_diff_command(opts.diff_cmd)
     lines_for_path.replace_tab_by = opts.replace_tab_by
+    Collection.ignore_names = tuple(opts.ignore_name)
     left, right = map(get_remote_file, (left, right))
     if os.path.isdir(left) != os.path.isdir(right):
         raise SystemExit('The items to be diffed should both be either directories or files. Comparing a directory to a file is not valid.')
@@ -686,6 +708,7 @@ elif __name__ == '__doc__':
     cd['usage'] = usage
     cd['options'] = OPTIONS
     cd['help_text'] = help_text
+    cd['args_completion'] = CompletionSpec.from_string('type:file mime:text/* mime:image/* group:"Text and image files"')
 elif __name__ == '__conf__':
     from .options.definition import definition
     sys.options_definition = definition  # type: ignore
