@@ -22,27 +22,16 @@ from base64 import standard_b64decode, standard_b64encode
 from contextlib import contextmanager, suppress
 from getpass import getuser
 from select import select
-from typing import (
-    Any, Callable, Dict, Iterator, List, NoReturn, Optional, Sequence, Set,
-    Tuple, Union, cast
-)
+from typing import Any, Callable, Dict, Iterator, List, NoReturn, Optional, Sequence, Set, Tuple, Union, cast
 
-from kitty.constants import (
-    cache_dir, runtime_dir, shell_integration_dir, ssh_control_master_template,
-    str_version, terminfo_dir
-)
+from kitty.constants import cache_dir, runtime_dir, shell_integration_dir, ssh_control_master_template, str_version, terminfo_dir
 from kitty.shell_integration import as_str_literal
 from kitty.shm import SharedMemory
 from kitty.types import run_once
-from kitty.utils import (
-    SSHConnectionData, expandvars, resolve_abs_or_config_path,
-    set_echo as turn_off_echo
-)
+from kitty.utils import SSHConnectionData, expandvars, resolve_abs_or_config_path
+from kitty.utils import set_echo as turn_off_echo
 
-from ..tui.operations import (
-    RESTORE_PRIVATE_MODE_VALUES, SAVE_PRIVATE_MODE_VALUES, Mode,
-    restore_colors, save_colors, set_mode
-)
+from ..tui.operations import RESTORE_PRIVATE_MODE_VALUES, SAVE_PRIVATE_MODE_VALUES, Mode, restore_colors, save_colors, set_mode
 from ..tui.utils import kitty_opts, running_in_tmux
 from .config import init_config
 from .copy import CopyInstruction
@@ -62,7 +51,7 @@ def read_data_from_shared_memory(shm_name: str) -> Any:
         if shm.stats.st_uid != os.geteuid() or shm.stats.st_gid != os.getegid():
             raise ValueError('Incorrect owner on pwfile')
         mode = stat.S_IMODE(shm.stats.st_mode)
-        if mode != stat.S_IREAD:
+        if mode != stat.S_IREAD | stat.S_IWRITE:
             raise ValueError('Incorrect permissions on pwfile')
         return json.loads(shm.read_data_with_size())
 
@@ -172,7 +161,8 @@ def make_tarfile(ssh_opts: SSHOptions, base_env: Dict[str, str], compression: st
     if ssh_opts.remote_kitty != 'no':
         env['KITTY_REMOTE'] = ssh_opts.remote_kitty
     if os.environ.get('KITTY_PUBLIC_KEY'):
-        env['KITTY_PUBLIC_KEY'] = os.environ['KITTY_PUBLIC_KEY']
+        env.pop('KITTY_PUBLIC_KEY', None)
+        literal_env['KITTY_PUBLIC_KEY'] = os.environ['KITTY_PUBLIC_KEY']
     env_script = serialize_env(literal_env, env, base_env, for_python=compression != 'gz')
     buf = io.BytesIO()
     with tarfile.open(mode=f'w:{compression}', fileobj=buf, encoding='utf-8') as tf:
@@ -192,6 +182,7 @@ def make_tarfile(ssh_opts: SSHOptions, base_env: Dict[str, str], compression: st
             arcname = 'home/' + rd + '/kitty'
             add_data_as_file(tf, arcname + '/version', str_version.encode('ascii'))
             tf.add(shell_integration_dir + '/ssh/kitty', arcname=arcname + '/bin/kitty', filter=normalize_tarinfo)
+            tf.add(shell_integration_dir + '/ssh/kitten', arcname=arcname + '/bin/kitten', filter=normalize_tarinfo)
         tf.add(f'{terminfo_dir}/kitty.terminfo', arcname='home/.terminfo/kitty.terminfo', filter=normalize_tarinfo)
         tf.add(glob.glob(f'{terminfo_dir}/*/xterm-kitty')[0], arcname='home/.terminfo/x/xterm-kitty', filter=normalize_tarinfo)
     return buf.getvalue()
@@ -622,9 +613,7 @@ def drain_potential_tty_garbage(p: 'subprocess.Popen[bytes]', data_request: str)
 def change_colors(color_scheme: str) -> bool:
     if not color_scheme:
         return False
-    from kittens.themes.collection import (
-        NoCacheFound, load_themes, text_as_opts
-    )
+    from kittens.themes.collection import NoCacheFound, load_themes, text_as_opts
     from kittens.themes.main import colors_as_escape_codes
     if color_scheme.endswith('.conf'):
         conf_file = resolve_abs_or_config_path(color_scheme)
@@ -753,9 +742,9 @@ def main(args: List[str]) -> None:
 
 if __name__ == '__main__':
     main(sys.argv)
-elif __name__ == '__completer__':
-    from .completion import complete
-    setattr(sys, 'kitten_completer', complete)
+elif __name__ == '__wrapper_of__':
+    cd = sys.cli_docs  # type: ignore
+    cd['wrapper_of'] = 'ssh'
 elif __name__ == '__conf__':
     from .options.definition import definition
     sys.options_definition = definition  # type: ignore
