@@ -7,15 +7,14 @@ import sys
 import traceback
 from enum import Enum, auto
 from gettext import gettext as _
-from typing import (
-    Any, Callable, Dict, Iterable, Iterator, List, Optional, Tuple, Union
-)
+from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
-from kitty.cli import create_default_opts, parse_args
+from kitty.cli import CompletionSpec, create_default_opts, parse_args
 from kitty.cli_stub import ThemesCLIOptions
 from kitty.config import cached_values_for
 from kitty.constants import config_dir
 from kitty.fast_data_types import truncate_point_for_length, wcswidth
+from kitty.options.types import Options as KittyOptions
 from kitty.rgb import color_as_sharp, color_from_int
 from kitty.typing import KeyEventType
 from kitty.utils import ScreenSize
@@ -23,7 +22,7 @@ from kitty.utils import ScreenSize
 from ..tui.handler import Handler
 from ..tui.line_edit import LineEdit
 from ..tui.loop import Loop
-from ..tui.operations import color_code, styled
+from ..tui.operations import color_code, set_default_colors, styled
 from .collection import MARK_AFTER, NoCacheFound, Theme, Themes, load_themes
 
 separator = '║'
@@ -37,7 +36,7 @@ def limit_length(text: str, limit: int = 32) -> str:
     x = truncate_point_for_length(text, limit - 1)
     if x >= len(text):
         return text
-    return text[:x] + '…'
+    return f'{text[:x]}…'
 
 
 class State(Enum):
@@ -68,6 +67,10 @@ def create_recent_filter(names: Iterable[str]) -> Callable[[Theme], bool]:
     return recent_filter
 
 
+def user_filter(q: Theme) -> bool:
+    return q.is_user_defined
+
+
 def mark_shortcut(text: str, acc: str) -> str:
     acc_idx = text.lower().index(acc.lower())
     return text[:acc_idx] + styled(text[acc_idx], underline='straight', bold=True, fg_intense=True) + text[acc_idx+1:]
@@ -132,6 +135,17 @@ class ThemesList:
         return self.themes[self.current_idx]
 
 
+def colors_as_escape_codes(o: KittyOptions) -> str:
+    ans = set_default_colors(
+        fg=o.foreground, bg=o.background, cursor=o.cursor, select_bg=o.selection_background, select_fg=o.selection_foreground
+    )
+    cmds = []
+    for i in range(256):
+        col = color_as_sharp(color_from_int(o.color_table[i]))
+        cmds.append(f'{i};{col}')
+    return ans + '\033]4;' + ';'.join(cmds) + '\033\\'
+
+
 class ThemesHandler(Handler):
 
     def __init__(self, cached_values: Dict[str, Any], cli_opts: ThemesCLIOptions) -> None:
@@ -141,12 +155,13 @@ class ThemesHandler(Handler):
         self.report_traceback_on_exit: Optional[str] = None
         self.filter_map: Dict[str, Callable[[Theme], bool]] = {
             'dark': dark_filter, 'light': light_filter, 'all': all_filter,
-            'recent': create_recent_filter(self.cached_values.get('recent', ()))
+            'recent': create_recent_filter(self.cached_values.get('recent', ())),
+            'user': user_filter
         }
         self.themes_list = ThemesList()
         self.colors_set_once = False
         self.line_edit = LineEdit()
-        self.tabs = tuple('all dark light recent'.split())
+        self.tabs = tuple('all dark light recent user'.split())
         self.quit_on_next_key_release = -1
 
     def update_recent(self) -> None:
@@ -163,7 +178,7 @@ class ThemesHandler(Handler):
         self.cmd.save_colors()
         self.cmd.set_line_wrapping(False)
         self.cmd.set_window_title('Choose a theme for kitty')
-        self.cmd.set_cursor_shape('bar')
+        self.cmd.set_cursor_shape('beam')
 
     def initialize(self) -> None:
         self.init_terminal_state()
@@ -195,15 +210,9 @@ class ThemesHandler(Handler):
             o = self.themes_list.current_theme.kitty_opts
         else:
             o = create_default_opts()
-        self.cmd.set_default_colors(
-            fg=o.foreground, bg=o.background, cursor=o.cursor, select_bg=o.selection_background, select_fg=o.selection_foreground
-        )
         self.current_opts = o
-        cmds = []
-        for i in range(256):
-            col = color_as_sharp(color_from_int(o.color_table[i]))
-            cmds.append(f'{i};{col}')
-        self.print(end='\033]4;' + ';'.join(cmds) + '\033\\')
+        cmd = colors_as_escape_codes(o)
+        self.write(cmd)
         return True
 
     def redraw_after_category_change(self) -> None:
@@ -224,8 +233,11 @@ class ThemesHandler(Handler):
             self.redraw_after_category_change()
 
         def fetch() -> None:
+            from urllib.error import URLError
             try:
                 themes: Union[Themes, str] = load_themes(self.cli_opts.cache_age)
+            except URLError as e:
+                themes = f'Could not download themes, check your internet connection. Error: {e}'
             except Exception:
                 themes = format_traceback('Failed to download themes')
             self.asyncio_loop.call_soon_threadsafe(fetching_done, themes)
@@ -332,7 +344,7 @@ class ThemesHandler(Handler):
         for line, width, is_current in self.themes_list.lines(num_rows):
             num_rows -= 1
             if is_current:
-                line = line.replace(MARK_AFTER, '\033[' + color_code('green') + 'm')
+                line = line.replace(MARK_AFTER, f'\033[{color_code("green")}m')
             self.cmd.styled('>' if is_current else ' ', fg='green')
             self.cmd.styled(line, bold=is_current, fg='green' if is_current else None)
             self.cmd.move_cursor_by(mw - width, 'right')
@@ -383,7 +395,7 @@ class ThemesHandler(Handler):
         if key_event.matches('esc') or key_event.matches_text('q'):
             self.quit_on_next_key_release = 0
             return
-        for cat in 'all dark light recent'.split():
+        for cat in self.tabs:
             if key_event.matches_text(cat[0]) or key_event.matches(f'alt+{cat[0]}'):
                 if cat != self.current_category:
                     self.current_category = cat
@@ -433,7 +445,7 @@ class ThemesHandler(Handler):
     def draw_accepting_screen(self) -> None:
         name = self.themes_list.current_theme.name
         name = styled(name, bold=True, fg="green")
-        kc = styled('kitty.conf', italic=True)
+        kc = styled(self.cli_opts.config_file_name, italic=True)
 
         def ac(x: str) -> str:
             return styled(x, fg='red')
@@ -465,7 +477,7 @@ class ThemesHandler(Handler):
             self.quit_on_next_key_release = 0
             return
         if key_event.matches_text('m'):
-            self.themes_list.current_theme.save_in_conf(config_dir, self.cli_opts.reload_in)
+            self.themes_list.current_theme.save_in_conf(config_dir, self.cli_opts.reload_in, self.cli_opts.config_file_name)
             self.update_recent()
             self.quit_on_next_key_release = 0
             return
@@ -536,6 +548,15 @@ type=bool-set
 default=false
 When running non-interactively, dump the specified theme to STDOUT
 instead of changing kitty.conf.
+
+
+--config-file-name
+default=kitty.conf
+The name or path to the config file to edit. Relative paths are interpreted
+with respect to the kitty config directory. By default the kitty config file,
+kitty.conf is edited. This is most useful if you add :code:`include themes.conf`
+to your kitty.conf and then have the kitten operate only on :file:`themes.conf`,
+allowing :code:`kitty.conf` to remain unchanged.
 '''.format
 
 
@@ -559,7 +580,7 @@ def non_interactive(cli_opts: ThemesCLIOptions, theme_name: str) -> None:
     if cli_opts.dump_theme:
         print(theme.raw)
         return
-    theme.save_in_conf(config_dir, cli_opts.reload_in)
+    theme.save_in_conf(config_dir, cli_opts.reload_in, cli_opts.config_file_name)
 
 
 def main(args: List[str]) -> None:
@@ -598,3 +619,4 @@ elif __name__ == '__doc__':
     cd['usage'] = usage
     cd['options'] = OPTIONS
     cd['help_text'] = help_text
+    cd['args_completion'] = CompletionSpec.from_string('type:special group:complete_themes')
