@@ -120,6 +120,40 @@ set_maximum_wait(monotonic_t val) {
     if (val >= 0 && (val < maximum_wait || maximum_wait < 0)) maximum_wait = val;
 }
 
+#define KITTY_HANDLED_SIGNALS SIGINT, SIGHUP, SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2, 0
+
+static void
+mask_variadic_signals(int sentinel, ...) {
+    sigset_t signals;
+    sigemptyset(&signals);
+    va_list valist;
+    va_start(valist, sentinel);
+    while (true) {
+        int sig = va_arg(valist, int);
+        if (sig == sentinel) break;
+        sigaddset(&signals, sig);
+    }
+    va_end(valist);
+#ifdef HAS_SIGNAL_FD
+    sigprocmask(SIG_BLOCK, &signals, NULL);
+#else
+    struct sigaction act = {.sa_handler=SIG_IGN, .sa_flags=SA_RESTART, .sa_mask = signals};
+    va_start(valist, sentinel);
+    while (true) {
+        int sig = va_arg(valist, int);
+        if (sig == sentinel) break;
+        sigaction(sig, &act, NULL);
+    }
+    va_end(valist);
+#endif
+}
+
+static PyObject*
+mask_kitty_signals_process_wide(PyObject *self UNUSED, PyObject *a UNUSED) {
+    mask_variadic_signals(0, KITTY_HANDLED_SIGNALS);
+    Py_RETURN_NONE;
+}
+
 static PyObject *
 new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
     ChildMonitor *self;
@@ -138,7 +172,7 @@ new(PyTypeObject *type, PyObject *args, PyObject UNUSED *kwds) {
         return NULL;
     }
     self = (ChildMonitor *)type->tp_alloc(type, 0);
-    if (!init_loop_data(&self->io_loop_data, SIGINT, SIGHUP, SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2, 0)) return PyErr_SetFromErrno(PyExc_OSError);
+    if (!init_loop_data(&self->io_loop_data, KITTY_HANDLED_SIGNALS)) return PyErr_SetFromErrno(PyExc_OSError);
     self->talk_fd = talk_fd;
     self->listen_fd = listen_fd;
     self->prewarm_fd = prewarm_fd;
@@ -472,17 +506,30 @@ parse_input(ChildMonitor *self) {
     return input_read;
 }
 
-static void
+static bool
 mark_child_for_close(ChildMonitor *self, id_type window_id) {
+    bool found = false;
     children_mutex(lock);
     for (size_t i = 0; i < self->count; i++) {
         if (children[i].id == window_id) {
             children[i].needs_removal = true;
+            found = true;
             break;
         }
     }
+    if (!found) {
+        for (size_t i = 0; i < add_queue_count; i++) {
+            if (add_queue[i].id == window_id) {
+                add_queue[i].needs_removal = true;
+                found = true;
+                break;
+            }
+        }
+
+    }
     children_mutex(unlock);
     wakeup_io_loop(self, false);
+    return found;
 }
 
 
@@ -491,8 +538,8 @@ mark_for_close(ChildMonitor *self, PyObject *args) {
 #define mark_for_close_doc "Mark a child to be removed from the child monitor"
     id_type window_id;
     if (!PyArg_ParseTuple(args, "K", &window_id)) return NULL;
-    mark_child_for_close(self, window_id);
-    Py_RETURN_NONE;
+    if (mark_child_for_close(self, window_id)) { Py_RETURN_TRUE; }
+    Py_RETURN_FALSE;
 }
 
 static bool
@@ -1074,6 +1121,10 @@ process_cocoa_pending_actions(void) {
     if (cocoa_pending_actions[TOGGLE_MACOS_SECURE_KEYBOARD_ENTRY]) { call_boss(toggle_macos_secure_keyboard_entry, NULL); }
     if (cocoa_pending_actions[TOGGLE_FULLSCREEN]) { call_boss(toggle_fullscreen, NULL); }
     if (cocoa_pending_actions[OPEN_KITTY_WEBSITE]) { call_boss(open_kitty_website, NULL); }
+    if (cocoa_pending_actions[HIDE]) { call_boss(hide_macos_app, NULL); }
+    if (cocoa_pending_actions[HIDE_OTHERS]) { call_boss(hide_macos_other_apps, NULL); }
+    if (cocoa_pending_actions[MINIMIZE]) { call_boss(minimize_macos_window, NULL); }
+    if (cocoa_pending_actions[QUIT]) { call_boss(quit, NULL); }
     if (cocoa_pending_actions_data.wd) {
         if (cocoa_pending_actions[NEW_OS_WINDOW_WITH_WD]) { call_boss(new_os_window_with_wd, "sO", cocoa_pending_actions_data.wd, Py_True); }
         if (cocoa_pending_actions[NEW_TAB_WITH_WD]) { call_boss(new_tab_with_wd, "sO", cocoa_pending_actions_data.wd, Py_True); }
@@ -1350,6 +1401,20 @@ reap_children(ChildMonitor *self, bool enable_close_on_child_death) {
     }
 }
 
+#ifdef KITTY_PRINT_BYTES_SENT_TO_CHILD
+static void
+print_text(const unsigned char *text, ssize_t sz) {
+    for (ssize_t i = 0; i < sz; i++) {
+        unsigned char ch = text[i];
+        if (32 <= ch && ch < 127) {
+            if (ch == '\\') fprintf(stderr, "%c", ch);
+            fprintf(stderr, "%c", ch);
+        } else fprintf(stderr, "\\x%02x", ch);
+    }
+}
+#endif
+
+
 static void
 write_to_child(int fd, Screen *screen) {
     size_t written = 0;
@@ -1357,7 +1422,15 @@ write_to_child(int fd, Screen *screen) {
     screen_mutex(lock, write);
     while (written < screen->write_buf_used) {
         ret = write(fd, screen->write_buf + written, screen->write_buf_used - written);
-        if (ret > 0) { written += ret; }
+#ifdef KITTY_PRINT_BYTES_SENT_TO_CHILD
+        fprintf(stderr, "Wrote: %zd bytes: ", ret);
+#endif
+        if (ret > 0) {
+#ifdef KITTY_PRINT_BYTES_SENT_TO_CHILD
+            print_text(screen->write_buf + written, ret);
+#endif
+            written += ret;
+        }
         else if (ret == 0) {
             // could mean anything, ignore
             break;
@@ -1367,6 +1440,9 @@ write_to_child(int fd, Screen *screen) {
             perror("Call to write() to child fd failed, discarding data.");
             written = screen->write_buf_used;
         }
+#ifdef KITTY_PRINT_BYTES_SENT_TO_CHILD
+        fprintf(stderr, "\n");
+#endif
     }
     if (written) {
         screen->write_buf_used -= written;
@@ -1829,6 +1905,7 @@ static PyMethodDef module_methods[] = {
     METHODB(monitor_pid, METH_VARARGS),
     METHODB(send_data_to_peer, METH_VARARGS),
     METHODB(cocoa_set_menubar_title, METH_VARARGS),
+    METHODB(mask_kitty_signals_process_wide, METH_NOARGS),
     {"sigqueue", (PyCFunction)sig_queue, METH_VARARGS, ""},
     {NULL}  /* Sentinel */
 };
