@@ -5,17 +5,11 @@ import os
 import sys
 from collections import defaultdict
 from contextlib import contextmanager, suppress
-from typing import (
-    TYPE_CHECKING, DefaultDict, Dict, Generator, List, Optional, Sequence,
-    Tuple
-)
+from typing import TYPE_CHECKING, DefaultDict, Dict, Generator, List, Optional, Sequence, Tuple
 
 import kitty.fast_data_types as fast_data_types
 
-from .constants import (
-    handled_signals, is_freebsd, is_macos, kitty_base_dir, shell_path,
-    terminfo_dir
-)
+from .constants import handled_signals, is_freebsd, is_macos, kitten_exe, kitty_base_dir, shell_path, terminfo_dir, wrapped_kitten_names
 from .types import run_once
 from .utils import log_error, which
 
@@ -28,11 +22,10 @@ if TYPE_CHECKING:
 
 
 if is_macos:
-    from kitty.fast_data_types import (
-        cmdline_of_process as cmdline_, cwd_of_process as _cwd,
-        environ_of_process as _environ_of_process,
-        process_group_map as _process_group_map
-    )
+    from kitty.fast_data_types import cmdline_of_process as cmdline_
+    from kitty.fast_data_types import cwd_of_process as _cwd
+    from kitty.fast_data_types import environ_of_process as _environ_of_process
+    from kitty.fast_data_types import process_group_map as _process_group_map
 
     def cwd_of_process(pid: int) -> str:
         return os.path.realpath(_cwd(pid))
@@ -192,16 +185,31 @@ class ProcessDesc(TypedDict):
     cmdline: Optional[Sequence[str]]
 
 
-def is_prewarmable(argv: Sequence[str]) -> bool:
+def is_prewarmable(argv: List[str]) -> Tuple[bool, List[str]]:
     if len(argv) < 3 or os.path.basename(argv[0]) != 'kitty':
-        return False
-    if argv[1][:1] not in '@+':
-        return False
-    if argv[1][0] == '@':
-        return True
+        return False, argv
+    if argv[1][:1] != '+':
+        return False, argv
+    sw = ''
     if argv[1] == '+':
-        return argv[2] != 'open'
-    return argv[1] != '+open'
+        which = argv[2]
+        if len(argv) > 3:
+            sw = argv[3]
+    else:
+        which = argv[1][1:]
+        if len(argv) > 2:
+            sw = argv[2]
+    if which == 'open':
+        return False, argv
+    if which == 'kitten' and sw in wrapped_kitten_names():
+        argv = list(argv)
+        argv[0] = kitten_exe()
+        if argv[1] == '+':
+            del argv[1:3]
+        else:
+            del argv[1]
+        return False, argv
+    return True, argv
 
 
 @run_once
@@ -288,7 +296,7 @@ class Child:
         self.forked = True
         master, slave = openpty()
         stdin, self.stdin = self.stdin, None
-        self.is_prewarmed = is_prewarmable(self.argv)
+        self.is_prewarmed, self.argv = is_prewarmable(self.argv)
         if not self.is_prewarmed:
             ready_read_fd, ready_write_fd = os.pipe()
             os.set_inheritable(ready_write_fd, False)
@@ -327,7 +335,7 @@ class Child:
         else:
             pid = fast_data_types.spawn(
                 self.final_exe, self.cwd, tuple(argv), env, master, slave, stdin_read_fd, stdin_write_fd,
-                ready_read_fd, ready_write_fd, tuple(handled_signals))
+                ready_read_fd, ready_write_fd, tuple(handled_signals), kitten_exe())
         os.close(slave)
         self.pid = pid
         self.child_fd = master
@@ -444,6 +452,15 @@ class Child:
                 return cwd_of_process(pid) or None
         return None
 
+    def get_foreground_exe(self, oldest: bool = False) -> Optional[str]:
+        with suppress(Exception):
+            pid = self.get_pid_for_cwd(oldest)
+            if pid is not None:
+                c = cmdline_of_pid(pid)
+                if c:
+                    return c[0]
+        return None
+
     @property
     def foreground_cwd(self) -> Optional[str]:
         return self.get_foreground_cwd()
