@@ -438,8 +438,7 @@ dispatch_osc(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
             START_DISPATCH
             DISPATCH_OSC_WITH_CODE(set_dynamic_color);
             END_DISPATCH
-        case 52:
-        case -52:
+        case 52: case -52: case 5522:
             START_DISPATCH
             DISPATCH_OSC_WITH_CODE(clipboard_control);
             if (code == -52) continue_osc_52(screen);
@@ -609,12 +608,12 @@ parse_sgr(Screen *screen, uint32_t *buf, unsigned int num, int *params, PyObject
                     case COLOR:
                     case COLOR1:
                     case COLOR3:
-                        REPORT_ERROR("Invalid SGR code containing disallowed character: %s", utf8(buf[i]));
+                        REPORT_ERROR("Invalid SGR code containing disallowed character: %s (U+%x)", utf8(buf[i]), buf[i]);
                         return;
                 }
                 break;
             default:
-                REPORT_ERROR("Invalid SGR code containing disallowed character: %s", utf8(buf[i]));
+                REPORT_ERROR("Invalid SGR code containing disallowed character: %s (U+%x)", utf8(buf[i]), buf[i]);
                 return;
         }
     }
@@ -1093,7 +1092,9 @@ dispatch_dcs(Screen *screen, PyObject DUMP_UNUSED *dump_callback) {
                 } else IF_SIMPLE_PREFIX("edit|", handle_remote_edit)
 #undef IF_SIMPLE_PREFIX
                 } else {
-                    REPORT_ERROR("Unrecognized DCS @ code: 0x%x", screen->parser_buf[1]);
+                    PyObject *tp = PyUnicode_FromKindAndData(PyUnicode_4BYTE_KIND, screen->parser_buf, screen->parser_buf_pos);
+                    REPORT_ERROR("Unrecognized DCS @ code: %s", tp ? PyUnicode_AsUTF8(tp) : "could not read");
+                    Py_XDECREF(tp);
                 }
             }
             break;
