@@ -332,9 +332,10 @@ cell_update_uniform_block(ssize_t vao_idx, Screen *screen, int uniform_buffer, c
         color_type cell_fg = rd->default_fg, cell_bg = rd->default_bg;
         index_type cell_color_x = screen->cursor->x;
         bool cursor_ok = screen->cursor->x < screen->columns && screen->cursor->y < screen->lines;
+        bool reversed = false;
         if (cursor_ok) {
             linebuf_init_line(screen->linebuf, screen->cursor->y);
-            colors_for_cell(screen->linebuf->line, screen->color_profile, &cell_color_x, &cell_fg, &cell_bg);
+            colors_for_cell(screen->linebuf->line, screen->color_profile, &cell_color_x, &cell_fg, &cell_bg, &reversed);
         }
         if (screen->color_profile->overridden.cursor_color.type == COLOR_IS_INDEX || screen->color_profile->overridden.cursor_color.type == COLOR_IS_RGB) {
             // since the program is controlling the cursor color we hope it has chosen one
@@ -571,32 +572,39 @@ set_cell_uniforms(float current_inactive_text_alpha, bool force) {
 }
 
 static GLfloat
-render_window_title(OSWindow *os_window, Screen *screen UNUSED, GLfloat xstart, GLfloat ystart, GLfloat width, Window *window, GLfloat left, GLfloat right) {
+render_a_bar(OSWindow *os_window, Screen *screen, const CellRenderData *crd, WindowBarData *bar, PyObject *title, bool along_bottom) {
+    GLfloat left = os_window->viewport_width * (crd->gl.xstart + 1.f) / 2.f;
+    GLfloat right = left + os_window->viewport_width * crd->gl.width / 2.f;
     unsigned bar_height = os_window->fonts_data->cell_height + 2;
     if (!bar_height || right <= left) return 0;
     unsigned bar_width = (unsigned)ceilf(right - left);
-    if (!window->title_bar_data.buf || window->title_bar_data.width != bar_width || window->title_bar_data.height != bar_height) {
-        free(window->title_bar_data.buf);
-        Py_CLEAR(window->title_bar_data.last_drawn_title_object_id);
-        window->title_bar_data.buf = malloc((size_t)4 * bar_width * bar_height);
-        if (!window->title_bar_data.buf) return 0;
-        window->title_bar_data.height = bar_height;
-        window->title_bar_data.width = bar_width;
+    if (!bar->buf || bar->width != bar_width || bar->height != bar_height) {
+        free(bar->buf);
+        bar->buf = malloc((size_t)4 * bar_width * bar_height);
+        if (!bar->buf) return 0;
+        bar->height = bar_height;
+        bar->width = bar_width;
+        bar->needs_render = true;
     }
-    static char title[2048] = {0};
-    if (window->title_bar_data.last_drawn_title_object_id != window->title) {
-        snprintf(title, arraysz(title), " %s", PyUnicode_AsUTF8(window->title));
+
+    if (bar->last_drawn_title_object_id != title || bar->needs_render) {
+        static char titlebuf[2048] = {0};
+        if (!title) return 0;
+        snprintf(titlebuf, arraysz(titlebuf), " %s", PyUnicode_AsUTF8(title));
 #define RGBCOL(which, fallback) ( 0xff000000 | colorprofile_to_color_with_fallback(screen->color_profile, screen->color_profile->overridden.which, screen->color_profile->configured.which, screen->color_profile->overridden.fallback, screen->color_profile->configured.fallback))
-        if (!draw_window_title(os_window, title, RGBCOL(highlight_fg, default_fg), RGBCOL(highlight_bg, default_bg), window->title_bar_data.buf, bar_width, bar_height)) return 0;
+        if (!draw_window_title(os_window, titlebuf, RGBCOL(highlight_fg, default_fg), RGBCOL(highlight_bg, default_bg), bar->buf, bar_width, bar_height)) return 0;
 #undef RGBCOL
-        window->title_bar_data.last_drawn_title_object_id = window->title;
-        Py_INCREF(window->title_bar_data.last_drawn_title_object_id);
+        Py_CLEAR(bar->last_drawn_title_object_id);
+        bar->last_drawn_title_object_id = title;
+        Py_INCREF(bar->last_drawn_title_object_id);
     }
     static ImageRenderData data = {.group_count=1};
-    xstart = clamp_position_to_nearest_pixel(xstart, os_window->viewport_width);
-    ystart = clamp_position_to_nearest_pixel(ystart, os_window->viewport_height);
+    GLfloat xstart, ystart;
+    xstart = clamp_position_to_nearest_pixel(crd->gl.xstart, os_window->viewport_width);
     GLfloat height_gl = gl_size(bar_height, os_window->viewport_height);
-    gpu_data_for_image(&data, xstart, ystart, xstart + width, ystart - height_gl);
+    if (along_bottom) ystart = crd->gl.ystart - crd->gl.height + height_gl;
+    else ystart = clamp_position_to_nearest_pixel(crd->gl.ystart, os_window->viewport_height);
+    gpu_data_for_image(&data, xstart, ystart, xstart + crd->gl.width, ystart - height_gl);
     if (!data.texture_id) { glGenTextures(1, &data.texture_id); }
     glBindTexture(GL_TEXTURE_2D, data.texture_id);
     glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
@@ -604,7 +612,7 @@ render_window_title(OSWindow *os_window, Screen *screen UNUSED, GLfloat xstart,
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bar_width, bar_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, window->title_bar_data.buf);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bar_width, bar_height, 0, GL_RGBA, GL_UNSIGNED_BYTE, bar->buf);
     set_cell_uniforms(1.f, false);
     bind_program(GRAPHICS_PROGRAM);
     send_graphics_data_to_gpu(1, os_window->gvao_idx, &data);
@@ -615,6 +623,26 @@ render_window_title(OSWindow *os_window, Screen *screen UNUSED, GLfloat xstart,
     return height_gl;
 }
 
+static void
+draw_hyperlink_target(OSWindow *os_window, Screen *screen, const CellRenderData *crd, Window *window) {
+    WindowBarData *bd = &window->url_target_bar_data;
+    if (bd->hyperlink_id_for_title_object != screen->current_hyperlink_under_mouse.id) {
+        bd->hyperlink_id_for_title_object = screen->current_hyperlink_under_mouse.id;
+        Py_CLEAR(bd->last_drawn_title_object_id);
+        const char *url = get_hyperlink_for_id(screen->hyperlink_pool, bd->hyperlink_id_for_title_object, true);
+        if (url == NULL) url = "";
+        bd->last_drawn_title_object_id = PyObject_CallMethod(global_state.boss, "sanitize_url_for_dispay_to_user", "s", url);
+        if (bd->last_drawn_title_object_id == NULL) { PyErr_Print(); return; }
+        bd->needs_render = true;
+    }
+    if (bd->last_drawn_title_object_id == NULL) return;
+    const bool along_bottom = screen->current_hyperlink_under_mouse.y < 3;
+    PyObject *ref = bd->last_drawn_title_object_id;
+    Py_INCREF(ref);
+    render_a_bar(os_window, screen, crd, &window->title_bar_data, bd->last_drawn_title_object_id, along_bottom);
+    Py_DECREF(ref);
+}
+
 static void
 draw_window_logo(ssize_t vao_idx, OSWindow *os_window, const WindowLogoRenderData *wl, const CellRenderData *crd) {
     if (os_window->live_resize.in_progress) return;
@@ -642,7 +670,7 @@ draw_window_number(OSWindow *os_window, Screen *screen, const CellRenderData *cr
     GLfloat title_bar_height = 0;
     size_t requested_height = (size_t)(os_window->viewport_height * crd->gl.height / 2.f);
     if (window->title && PyUnicode_Check(window->title) && (requested_height > (os_window->fonts_data->cell_height + 1) * 2)) {
-        title_bar_height = render_window_title(os_window, screen, crd->gl.xstart, crd->gl.ystart, crd->gl.width, window, left, right);
+        title_bar_height = render_a_bar(os_window, screen, crd, &window->title_bar_data, window->title, false);
     }
     GLfloat ystart = crd->gl.ystart, height = crd->gl.height, xstart = crd->gl.xstart, width = crd->gl.width;
     if (title_bar_height > 0) {
@@ -933,11 +961,12 @@ draw_cells(ssize_t vao_idx, ssize_t gvao_idx, const ScreenRenderData *srd, float
     }
 
     if (window && screen->display_window_char) draw_window_number(os_window, screen, &crd, window);
+    if (OPT(show_hyperlink_targets) && window && screen->current_hyperlink_under_mouse.id && !is_mouse_hidden(os_window)) draw_hyperlink_target(os_window, screen, &crd, window);
 }
 // }}}
 
 // Borders {{{
-enum BorderUniforms { BORDER_viewport, BORDER_background_opacity, BORDER_tint_opacity, BORDER_tint_premult, BORDER_default_bg, BORDER_active_border_color, BORDER_inactive_border_color, BORDER_bell_border_color, BORDER_tab_bar_bg, BORDER_tab_bar_margin_color, NUM_BORDER_UNIFORMS };
+enum BorderUniforms { BORDER_viewport, BORDER_background_opacity, BORDER_tint_opacity, BORDER_tint_premult, BORDER_colors, NUM_BORDER_UNIFORMS };
 static GLint border_uniform_locations[NUM_BORDER_UNIFORMS] = {0};
 
 static void
@@ -947,12 +976,7 @@ init_borders_program(void) {
         SET_LOC(background_opacity)
         SET_LOC(tint_opacity)
         SET_LOC(tint_premult)
-        SET_LOC(default_bg)
-        SET_LOC(active_border_color)
-        SET_LOC(inactive_border_color)
-        SET_LOC(bell_border_color)
-        SET_LOC(tab_bar_bg)
-        SET_LOC(tab_bar_margin_color)
+        SET_LOC(colors)
 #undef SET_LOC
 }
 
@@ -980,7 +1004,7 @@ draw_borders(ssize_t vao_idx, unsigned int num_border_rects, BorderRect *rect_bu
         draw_bg(w);
         BLEND_ONTO_OPAQUE;
         background_opacity = 1.0f;
-        tint_opacity = OPT(background_tint);
+        tint_opacity = OPT(background_tint) * OPT(background_tint_gaps);
         tint_premult = w->is_semi_transparent ? OPT(background_tint) : 1.0f;
     }
 
@@ -993,19 +1017,17 @@ draw_borders(ssize_t vao_idx, unsigned int num_border_rects, BorderRect *rect_bu
             if (borders_buf_address) memcpy(borders_buf_address, rect_buf, sz);
             unmap_vao_buffer(vao_idx, 0);
         }
-#define CV3(x) (((float)((x >> 16) & 0xff))/255.f), (((float)((x >> 8) & 0xff))/255.f), (((float)(x & 0xff))/255.f)
+        color_type default_bg = (num_visible_windows > 1 && !all_windows_have_same_bg) ? OPT(background) : active_window_bg;
+        GLuint colors[9] = {
+            default_bg, OPT(active_border_color), OPT(inactive_border_color), 0,
+            OPT(bell_border_color), OPT(tab_bar_background), OPT(tab_bar_margin_color),
+            w->tab_bar_edge_color.left, w->tab_bar_edge_color.right
+        };
+        glUniform1uiv(border_uniform_locations[BORDER_colors], arraysz(colors), colors);
         glUniform1f(border_uniform_locations[BORDER_background_opacity], background_opacity);
         glUniform1f(border_uniform_locations[BORDER_tint_opacity], tint_opacity);
         glUniform1f(border_uniform_locations[BORDER_tint_premult], tint_premult);
-        glUniform3f(border_uniform_locations[BORDER_active_border_color], CV3(OPT(active_border_color)));
-        glUniform3f(border_uniform_locations[BORDER_inactive_border_color], CV3(OPT(inactive_border_color)));
-        glUniform3f(border_uniform_locations[BORDER_bell_border_color], CV3(OPT(bell_border_color)));
-        glUniform3f(border_uniform_locations[BORDER_tab_bar_bg], CV3(OPT(tab_bar_background)));
-        glUniform3f(border_uniform_locations[BORDER_tab_bar_margin_color], CV3(OPT(tab_bar_margin_color)));
         glUniform2ui(border_uniform_locations[BORDER_viewport], viewport_width, viewport_height);
-        color_type default_bg = (num_visible_windows > 1 && !all_windows_have_same_bg) ? OPT(background) : active_window_bg;
-        glUniform3f(border_uniform_locations[BORDER_default_bg], CV3(default_bg));
-#undef CV3
         if (has_bgimage(w)) {
             if (w->is_semi_transparent) { BLEND_PREMULT; }
             else { BLEND_ONTO_OPAQUE_WITH_OPAQUE_OUTPUT; }
