@@ -4,6 +4,7 @@
 
 import os
 import stat
+import subprocess
 import sys
 import unittest
 from functools import partial
@@ -14,11 +15,15 @@ from . import BaseTest
 class TestBuild(BaseTest):
 
     def test_exe(self) -> None:
-        from kitty.constants import kitty_exe
+        from kitty.constants import kitten_exe, kitty_exe, str_version
         exe = kitty_exe()
         self.assertTrue(os.access(exe, os.X_OK))
         self.assertTrue(os.path.isfile(exe))
         self.assertIn('kitty', os.path.basename(exe))
+        exe = kitten_exe()
+        self.assertTrue(os.access(exe, os.X_OK))
+        self.assertTrue(os.path.isfile(exe))
+        self.assertIn(str_version, subprocess.check_output([exe, '--version']).decode())
 
     def test_loading_extensions(self) -> None:
         import kitty.fast_data_types as fdt
@@ -52,9 +57,7 @@ class TestBuild(BaseTest):
         self.assertGreater(len(names), 8)
 
     def test_filesystem_locations(self) -> None:
-        from kitty.constants import (
-            local_docs, logo_png_file, shell_integration_dir, terminfo_dir
-        )
+        from kitty.constants import local_docs, logo_png_file, shell_integration_dir, terminfo_dir
         zsh = os.path.join(shell_integration_dir, 'zsh')
         self.assertTrue(os.path.isdir(terminfo_dir), f'Terminfo dir: {terminfo_dir}')
         self.assertTrue(os.path.exists(logo_png_file), f'Logo file: {logo_png_file}')
@@ -65,7 +68,7 @@ class TestBuild(BaseTest):
             q = stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
             return mode & q == q
 
-        for x in ('kitty', 'askpass.py'):
+        for x in ('kitty', 'kitten', 'askpass.py'):
             x = os.path.join(shell_integration_dir, 'ssh', x)
             self.assertTrue(is_executable(x), f'{x} is not executable')
         if getattr(sys, 'frozen', False):
@@ -108,8 +111,9 @@ class TestBuild(BaseTest):
         self.ae(docs_url('#ref=issues-123'), 'https://github.com/kovidgoyal/kitty/issues/123')
 
     def test_launcher_ensures_stdio(self):
-        from kitty.constants import kitty_exe
         import subprocess
+
+        from kitty.constants import kitty_exe
         exe = kitty_exe()
         cp = subprocess.run([exe, '+runpy', f'''\
 import os, sys
