@@ -2,7 +2,10 @@
 # License: GPL v3 Copyright: 2017, Kovid Goyal <kovid at kovidgoyal.net>
 
 import argparse
+import base64
+import contextlib
 import datetime
+import glob
 import io
 import json
 import mimetypes
@@ -16,9 +19,9 @@ import sys
 import tempfile
 import time
 from contextlib import contextmanager, suppress
-from typing import IO, Any, Dict, Generator, Iterable, Optional, cast
-
-import requests
+from http.client import HTTPResponse, HTTPSConnection
+from typing import Any, Callable, Dict, Generator, Iterable, Optional, Tuple, Union
+from urllib.parse import urlencode, urlparse
 
 os.chdir(os.path.dirname(os.path.abspath(__file__)))
 docs_dir = os.path.abspath('docs')
@@ -74,11 +77,12 @@ def run_build(args: Any) -> None:
             call(cmd, echo=True)
 
     for x in ('64', '32', 'arm64'):
-        cmd = f'python ../bypy linux --arch {x} program'
-        run_with_retry(cmd)
-        call(f'python ../bypy linux --arch {x} shutdown', echo=True)
-    cmd = 'python ../bypy macos program --sign-installers --notarize'
-    run_with_retry(cmd)
+        prefix = f'python ../bypy linux --arch {x} '
+        run_with_retry(prefix + 'program --non-interactive')
+        call(prefix + 'shutdown', echo=True)
+    run_with_retry('python ../bypy macos program --sign-installers --notarize --non-interactive')
+    call('python ../bypy macos shutdown', echo=True)
+    call('./setup.py build-static-binaries')
 
 
 def run_tag(args: Any) -> None:
@@ -198,17 +202,17 @@ class ReadFileWithProgressReporting(io.FileIO):  # {{{
         def write(*args: str) -> None:
             print(*args, end='')
 
-        frac = float(self.tell()) / self._total
+        frac = int(self.tell() * 100 / self._total)
         mb_pos = self.tell() / float(1024**2)
         mb_tot = self._total / float(1024**2)
         kb_pos = self.tell() / 1024.0
         kb_rate = kb_pos / (time.monotonic() - self.start_time)
         bit_rate = kb_rate * 1024
         eta = int((self._total - self.tell()) / bit_rate) + 1
-        eta_m, eta_s = eta / 60, eta % 60
+        eta_m, eta_s = divmod(eta, 60)
         if sys.stdout.isatty():
             write(
-                f'\r\033[K\033[?7h {frac:%} {mb_pos:.1f}/{mb_tot:.1f}MB {kb_rate:.1f} KB/sec {eta_m} minutes, {eta_s} seconds left\033[?7l')
+                f'\r\033[K\033[?7h {frac}% {mb_pos:.1f}/{mb_tot:.1f}MB {kb_rate:.1f} KB/sec {eta_m} minutes, {eta_s} seconds left\033[?7l')
         if self.tell() >= self._total:
             t = int(time.monotonic() - self.start_time) + 1
             print(f'\nUpload took {t//60} minutes and {t%60} seconds at {kb_rate:.1f} KB/sec')
@@ -218,23 +222,7 @@ class ReadFileWithProgressReporting(io.FileIO):  # {{{
 # }}}
 
 
-class Base:  # {{{
-
-    def info(self, *args: Any, **kwargs: Any) -> None:
-        print(*args, **kwargs)
-        sys.stdout.flush()
-
-    def warn(self, *args: Any, **kwargs: Any) -> None:
-        print('\n' + '_' * 20, 'WARNING', '_' * 20)
-        print(*args, **kwargs)
-        print('_' * 50)
-        sys.stdout.flush()
-
-
-# }}}
-
-
-class GitHub(Base):  # {{{
+class GitHub:  # {{{
 
     API = 'https://api.github.com'
 
@@ -251,20 +239,75 @@ class GitHub(Base):  # {{{
             files, reponame, version, username, password, replace)
         self.current_tag_name = self.version if self.version == 'nightly' else f'v{self.version}'
         self.is_nightly = self.current_tag_name == 'nightly'
-        self.requests = s = requests.Session()
-        s.auth = (self.username, self.password)
-        s.headers.update({'Accept': 'application/vnd.github.v3+json'})
+        self.auth = 'Basic ' + base64.standard_b64encode(f'{self.username}:{self.password}'.encode()).decode()
         self.url_base = f'{self.API}/repos/{self.username}/{self.reponame}/releases'
 
-    def patch(self, url: str, fail_msg: str, **data: Any) -> None:
-        rdata = json.dumps(data)
-        try:
-            r = self.requests.patch(url, data=rdata)
-        except Exception:
-            time.sleep(15)
-            r = self.requests.patch(url, data=rdata)
-        if r.status_code != 200:
-            self.fail(r, fail_msg)
+    def info(self, *args: Any) -> None:
+        print(*args, flush=True)
+
+    def error(self, *args: Any) -> None:
+        print(*args, flush=True, file=sys.stderr)
+
+    def make_request(
+        self, url: str, data: Optional[Dict[str, Any]] = None, method:str = 'GET',
+        upload_data: Optional[ReadFileWithProgressReporting] = None,
+        params: Optional[Dict[str, str]] = None,
+    ) -> HTTPSConnection:
+        headers={
+            'Authorization': self.auth,
+            'Accept': 'application/vnd.github+json',
+            'User-Agent': 'kitty',
+        }
+        if params:
+            url += '?' + urlencode(params)
+        rdata: Optional[Union[bytes, io.FileIO]] = None
+        if data is not None:
+            rdata = json.dumps(data).encode('utf-8')
+            headers['Content-Type'] = 'application/json'
+            headers['Content-Length'] = str(len(rdata))
+        elif upload_data is not None:
+            rdata = upload_data
+            mime_type = mimetypes.guess_type(os.path.basename(str(upload_data.name)))[0] or 'application/octet-stream'
+            headers['Content-Type'] = mime_type
+            headers['Content-Length'] = str(upload_data._total)
+        purl = urlparse(url)
+        conn = HTTPSConnection(purl.netloc, timeout=60)
+        conn.request(method, url, body=rdata, headers=headers)
+        return conn
+
+    def make_request_with_retries(
+        self, url: str, data: Optional[Dict[str, str]] = None, method:str = 'GET',
+        num_tries: int = 2, sleep_between_tries: float = 15,
+        success_codes: Tuple[int, ...] = (200,),
+        failure_msg: str = 'Request failed',
+        return_data: bool = False,
+        upload_path: str = '',
+        params: Optional[Dict[str, str]] = None,
+        failure_callback: Callable[[HTTPResponse], None] = lambda r: None,
+    ) -> Any:
+        for i in range(num_tries):
+            try:
+                if upload_path:
+                    conn = self.make_request(url, method='POST', upload_data=ReadFileWithProgressReporting(upload_path), params=params)
+                else:
+                    conn = self.make_request(url, data, method, params=params)
+                with contextlib.closing(conn):
+                    r = conn.getresponse()
+                    if r.status in success_codes:
+                        return json.loads(r.read()) if return_data else None
+                    if i == num_tries -1 :
+                        self.fail(r, failure_msg)
+                    else:
+                        self.print_failed_response_details(r, failure_msg)
+                        failure_callback(r)
+            except Exception as e:
+                self.error(failure_msg, 'with error:', e)
+            self.error(f'Retrying after {sleep_between_tries} seconds')
+            time.sleep(sleep_between_tries)
+        return None
+
+    def patch(self, url: str, fail_msg: str, **data: str) -> None:
+        self.make_request_with_retries(url, data, method='PATCH', failure_msg=fail_msg)
 
     def update_nightly_description(self, release_id: int) -> None:
         url = f'{self.url_base}/{release_id}'
@@ -276,100 +319,111 @@ class GitHub(Base):  # {{{
             ' For how to install nightly builds, see: https://sw.kovidgoyal.net/kitty/binary/#customizing-the-installation'
         )
 
+    def delete_asset(self, url: str, fname: str) -> None:
+        self.make_request_with_retries(
+            url, method='DELETE', num_tries=5, sleep_between_tries=2,
+            success_codes=(204, 404),
+            failure_msg=f'Failed to delete {fname} from GitHub')
+
     def __call__(self) -> None:
+        # See https://docs.github.com/en/rest/releases/assets#upload-a-release-asset
         # self.clean_older_releases(releases)
         release = self.create_release()
         upload_url = release['upload_url'].partition('{')[0]
         asset_url = f'{self.url_base}/assets/{{}}'
         existing_assets = self.existing_assets(release['id'])
-        if self.is_nightly:
-            for fname in existing_assets:
-                self.info(f'Deleting {fname} from GitHub')
-                r = self.requests.delete(asset_url.format(existing_assets[fname]))
-                if r.status_code not in (204, 404):
-                    self.fail(r, f'Failed to delete {fname} from GitHub')
-            self.update_nightly_description(release['id'])
-        for path, desc in self.files.items():
-            self.info('')
+
+        def delete_asset(asset_id: str) -> None:
+            self.delete_asset(asset_url.format(asset_id), fname)
+
+        def upload_with_retries(path: str, desc: str, num_tries: int = 8, sleep_time: float = 60.0) -> None:
             fname = os.path.basename(path)
             if self.is_nightly:
                 fname = fname.replace(version, 'nightly')
             if fname in existing_assets:
                 self.info(f'Deleting {fname} from GitHub with id: {existing_assets[fname]}')
-                r = self.requests.delete(asset_url.format(existing_assets[fname]))
-                if r.status_code not in (204, 404):
-                    self.fail(r, f'Failed to delete {fname} from GitHub')
-            r = self.do_upload(upload_url, path, desc, fname)
-            if r.status_code != 201:
-                self.fail(r, f'Failed to upload file: {fname}')
-            self.patch(asset_url.format(r.json()['id']), f'Failed to set label for {fname}', name=fname, label=desc)
+                delete_asset(existing_assets.pop(fname))
+            params = {'name': fname, 'label': desc}
+
+            def handle_failure(r: HTTPResponse) -> None:
+                try:
+                    asset_id = json.loads(r.read())['id']
+                except Exception:
+                    try:
+                        asset_id = self.existing_assets(release['id'])[fname]
+                    except KeyError:
+                        asset_id = 0
+                if asset_id:
+                    self.info(f'Deleting {fname} from GitHub with id: {asset_id}')
+                    delete_asset(asset_id)
+
+
+            self.make_request_with_retries(
+                upload_url, upload_path=path, params=params, num_tries=num_tries, sleep_between_tries=sleep_time,
+                failure_msg=f'Failed to upload file: {fname}', success_codes=(201,), failure_callback=handle_failure
+            )
+
+        if self.is_nightly:
+            for fname in tuple(existing_assets):
+                self.info(f'Deleting {fname} from GitHub with id: {existing_assets[fname]}')
+                delete_asset(existing_assets.pop(fname))
+            self.update_nightly_description(release['id'])
+        for path, desc in self.files.items():
+            self.info('')
+            upload_with_retries(path, desc)
 
     def clean_older_releases(self, releases: Iterable[Dict[str, Any]]) -> None:
         for release in releases:
-            if release.get(
-                    'assets',
-                    None) and release['tag_name'] != self.current_tag_name:
+            if release.get('assets') and release['tag_name'] != self.current_tag_name:
                 self.info(f'\nDeleting old released installers from: {release["tag_name"]}')
                 for asset in release['assets']:
-                    r = self.requests.delete(
-                        f'{self.url_base}/assets/{asset["id"]}')
-                    if r.status_code != 204:
-                        self.fail(r, f'Failed to delete obsolete asset: {asset["name"]} for release: {release["tag_name"]}')
+                    self.delete_asset(
+                        f'{self.url_base}/assets/{asset["id"]}', asset['name'])
 
-    def do_upload(self, url: str, path: str, desc: str, fname: str) -> requests.Response:
-        mime_type = mimetypes.guess_type(fname)[0] or 'application/octet-stream'
-        self.info(f'Uploading to GitHub: {fname} ({mime_type})')
-        with ReadFileWithProgressReporting(path) as f:
-            return self.requests.post(
-                url,
-                headers={
-                    'Content-Type': mime_type,
-                    'Content-Length': str(f._total)
-                },
-                params={'name': fname},
-                data=cast(IO[bytes], f))
+    def print_failed_response_details(self, r: HTTPResponse, msg: str) -> None:
+        self.error(msg, f'\nStatus Code: {r.status} {r.reason}')
+        try:
+            jr = json.loads(r.read())
+        except Exception:
+            pass
+        else:
+            self.error('JSON from response:')
+            pprint.pprint(jr, stream=sys.stderr)
 
-    def fail(self, r: requests.Response, msg: str) -> None:
-        print(msg, f' Status Code: {r.status_code}', file=sys.stderr)
-        print('JSON from response:', file=sys.stderr)
-        pprint.pprint(dict(r.json()), stream=sys.stderr)
+    def fail(self, r: HTTPResponse, msg: str) -> None:
+        self.print_failed_response_details(r, msg)
         raise SystemExit(1)
 
-    def already_exists(self, r: requests.Response) -> bool:
-        error_code = r.json().get('errors', [{}])[0].get('code', None)
-        return bool(error_code == 'already_exists')
-
     def existing_assets(self, release_id: str) -> Dict[str, str]:
         url = f'{self.url_base}/{release_id}/assets'
-        r = self.requests.get(url)
-        if r.status_code != 200:
-            self.fail(r, 'Failed to get assets for release')
-        return {asset['name']: asset['id'] for asset in r.json()}
+        d = self.make_request_with_retries(url, failure_msg='Failed to get assets for release', return_data=True)
+        return {asset['name']: asset['id'] for asset in d}
 
     def create_release(self) -> Dict[str, Any]:
         ' Create a release on GitHub or if it already exists, return the existing release '
         # Check for existing release
         url = f'{self.url_base}/tags/{self.current_tag_name}'
-        r = self.requests.get(url)
-        if r.status_code == 200:
-            return dict(r.json())
+        with contextlib.closing(self.make_request(url)) as conn:
+            r = conn.getresponse()
+            if r.status == 200:
+                return {str(k): v for k, v in json.loads(r.read()).items()}
         if self.is_nightly:
-            raise SystemExit('No existing nightly release found on GitHub')
-        r = self.requests.post(
-            self.url_base,
-            data=json.dumps({
-                'tag_name': self.current_tag_name,
-                'target_commitish': 'master',
-                'name': f'version {self.version}',
-                'body': f'Release version {self.version}.'
-                ' For changelog, see https://sw.kovidgoyal.net/kitty/changelog/#detailed-list-of-changes'
-                ' GPG key used for signing tarballs is: https://calibre-ebook.com/signatures/kovid.gpg',
-                'draft': False,
-                'prerelease': False
-            }))
-        if r.status_code != 201:
-            self.fail(r, f'Failed to create release for version: {self.version}')
-        return dict(r.json())
+            self.fail(r, 'No existing nightly release found on GitHub')
+        data = {
+            'tag_name': self.current_tag_name,
+            'target_commitish': 'master',
+            'name': f'version {self.version}',
+            'body': f'Release version {self.version}.'
+            ' For changelog, see https://sw.kovidgoyal.net/kitty/changelog/#detailed-list-of-changes'
+            ' GPG key used for signing tarballs is: https://calibre-ebook.com/signatures/kovid.gpg',
+            'draft': False,
+            'prerelease': False
+        }
+        with contextlib.closing(self.make_request(self.url_base, method='POST', data=data)) as conn:
+            r = conn.getresponse()
+            if r.status != 201:
+                self.fail(r, f'Failed to create release for version: {self.version}')
+            return {str(k): v for k, v in json.loads(r.read()).items()}
 # }}}
 
 
@@ -393,6 +447,17 @@ def files_for_upload() -> Dict[str, str]:
             raise SystemExit(f'The installer {path} does not exist')
         files[path] = desc
         signatures[path] = f'GPG signature for {desc}'
+    b = len(files)
+    for path in glob.glob('build/static/kitten-*'):
+        if path.endswith('.sig'):
+            continue
+        path = os.path.abspath(path)
+        exe_name = os.path.basename(path)
+        files[path] = f'Static {exe_name} executable'
+        signatures[path] = f'GPG signature for static {exe_name} executable'
+    if len(files) == b:
+        raise SystemExit('No static binaries found')
+
     files[f'build/kitty-{version}.tar.xz'] = 'Source code'
     files[f'build/kitty-{version}.tar.xz.sig'] = 'Source code GPG signature'
     for path, desc in signatures.items():
